﻿/* FMmarkers.jsx ------ UTF-8 ------------------------------------------------------------------------
                Handle markers of all sorts, in particular index markers
  Documentation www.daube.ch/docu/fmaker89.html
  Comment       To be placed in Startup folder of user area
                Normally compiled using the compiler by running !_Make-distribution
                PrepareInstallation is assumed to be on desktop together with project installation-directory
                This installation program resides in \InstallThis (1 level below PrepareInstallation)
                This utility program resides in \InstallThis\scripts (2 levels below PrepareInstallation)
  Tasks         • The utility script FMmarkers resides in \InstallThis\cripts and may be ahk and/or jsx
                • Application scripts will be copied into the FM-user-area and optionally the Adobe script folder
  Reference     Based on FMcalc and FMvars
                Traduction française avec l'aide de https://www.deepl.com/translator par Klaus Daube
  Copyleft      2021 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
  History       2021-09-18 started
*/ ; // ===========================================================================================

//@target framemaker

if (typeof KLD_M == "undefined") {
  KLD_M = {};                                     // Global script object for this suite
}
//@include FMmarkers\FMmarkers_Globals.jsx        // all declarations, includes FMmarkers_Texts.jsx
//@include FMmarkers\FMmarkers_Ancillary.jsx      // ancillary functions
//@include FMmarkers\FMmarkers_FunM.jsx           // Call back and other functions
////@include FMmarkers\FMmarkers_PalI.jsx           // incl. button functions etc. →→→→→→ version 2
//@include FMmarkers\FMmarkers_PalM.jsx           // incl. button functions etc.
//         FMmarkers\FMmarkers_en.xml             // Read by SwitchUIlanguageX
//         FMmarkers\FMmarkers_de.xml             // See there
//         FMmarkers\FMmarkers_fr.xml             //

KLD_M.Main = function () { //=== ==================================================================
/*            Initial function of this script suite.
Called by     Statement at end of this module
Calling       DebugMenu, FindReplInDoc, SetupFMmarkers, SetUpMenus, SetupNotifications
Attention     With functions as objects the call to Main must be at end of this module.
History       2020-12-29
*/
var bIsOK = false;
  KLD_M.FM_VERSION = app.VersionMajor;
  Console (KLD_M.APP_NAME + " v" + KLD_M.APP_VERSION);
  KLD_M.APP_LANG = KLD_M.SwitchUIlanguageX (KLD_M.APP_NAME); // etablish the UI texts
  bIsOK = KLD_M.SetupFMmarkers ();                // get inital data / terminate on error
  if (!bIsOK) {return false;}
// --- For debugging you need to switch statements (ESTK not working when menus active)
//                              real work       debugging
//KLD_M.SetupNotifications ();  // uncomment    comment --- not used (at least for now) 2021-11-17
  KLD_M.SetUpMenus();           // uncomment    uncomment
//KLD_M.DebugMenu ();           // comment      uncomment
//KLD_M.HandleAny ();           // Call directly during initial tests
//KLD_M.HandleIndex ();         // Call directly during initial tests
} ; // --- end Main  ------------------------------------------------------------------------------

KLD_M.SetUpMenus = function () { //=== Define the menu items for document and book ================
/*            Set up the menu and short cuts
Called by     Main
Calling       UpdateMenus
Attention     This function MUST NOT be named DefineMenu - because this is a reserved function!
Comment       There is no menu "Special" in books => EditMenu used instead for both cases
              For a list of DDD used … see E:\_DDDprojects\FM-GUI-modifications\ESC-sequences\FM-ESC-sequences.fm
              The command location in the menu is only observed in FM-16 upwards.
Reference     Klaus Göbel at https://forums.adobe.com/thread/2236201
              (My method with kb-labels after \t in the menu entry defintions does not work in FM-14)
History       2020-11-20
*/
var docFMmarkMenu, menuLocation, nearbyCmd, oMenus = {}, oCmd = {};
// --- Menu entries (could be integrated in to command definitions as strings)
  oMenus.MenuMain    = KLD_M.UItxt.SetUpMenus.@menu00.toString() + KLD_M.APP_VERSION;
  oMenus.MenuDocu    = KLD_M.UItxt.SetUpMenus.@menu01.toString();
  oMenus.HandleAny   = KLD_M.UItxt.SetUpMenus.@menu02.toString();
//oMenus.HandleIndex = KLD_M.UItxt.SetUpMenus.@menu03.toString();
// --- Command definition for documents and books
  oCmd.MenuMain    = DefineCommand(0,"FMmarker_Main", oMenus.MenuMain,   "");
  oCmd.MenuDocu    = DefineCommand(1,"FMmarker_Docu", oMenus.MenuDocu,  KLD_M.UItxt.SetUpMenus.@sc01.toString());
  oCmd.HandleAny   = DefineCommand(2,"FMmarker_Any",  oMenus.HandleAny, KLD_M.UItxt.SetUpMenus.@sc02.toString());
//oCmd.HandleIndex = DefineCommand(3,"FMmarker_Index",oMenus.HandleAny, KLD_M.UItxt.SetUpMenus.@sc03.toString());
// --- Shortcut labels
  oCmd.MenuDocu.KeyboardShortcutLabel    = KLD_M.UItxt.SetUpMenus.@scl01.toString();
  oCmd.HandleAny.KeyboardShortcutLabel   = KLD_M.UItxt.SetUpMenus.@scl02.toString();
//oCmd.HandleIndex.KeyboardShortcutLabel = KLD_M.UItxt.SetUpMenus.@scl03.toString();
// --- Assign commands to menus
  if (KLD_M.FM_VERSION > 13) {
    menuLocation   = app.GetNamedMenu("InsertMenu");
  } else {
    menuLocation   = app.GetNamedMenu("SpecialMenu");
  }
  nearbyCmd = app.GetNamedCommand("Marker");      // define placement, works only for FM ≥ 16
  oCmd.MenuMain.PrevMenuItemInMenu = nearbyCmd;
  docFMmarkMenu  = menuLocation.DefineAndAddMenu("!FMmarkMain", oMenus.MenuMain); // ! required
  docFMmarkMenu.AddCommandToMenu (oCmd.MenuDocu);
  docFMmarkMenu.AddCommandToMenu (oCmd.HandleAny);

  UpdateMenus();                                  // refresh and actually make the new menu appear
} //--- end SetUpMenus ----------------------------------------------------------------------------

function Command (cmd) { // =======================================================================
/*            Assign functions to menu commands
Arguments     cmd            reference number of command from the menu definition
Returns       -
Called by     SetUpMenus, DebugMenu
Calling       FindReplInDoc, FindReplInBook
Attention     This functions can only have this name (Command) and must use the classic syntax
              function Command (cmd) --- not --- Command = function (cmd) !
History       2020-11-19 tested 'as implemented'
*/
  switch (cmd) {
    case 1:
      KLD_M.OpenHelpFile();
      break;
    case 2:
      KLD_M.HandleAny();
      break;
    case 3:
//    KLD_M.HandleIndex();
      break;
  }
} //--- end Command -------------------------------------------------------------------------------

KLD_M.HandleAny = function () { //=================================================================
/*            Handle any marker for documents
Called by     Command
Calling       Message, PaletteFindRepl, SaveCurrentLocation, RestoreLocation
History       2020-12-29
*/
var  oLocation;

  KLD_M.oCurrentDoc = app.ActiveDoc;
  if (!KLD_M.oCurrentDoc.ObjectValid()){
    KLD_M.Message ("E", KLD_M.UItxt.General.@DocRequired, "HandleAny");
    return;
  }
  KLD_M.bDocActive = true;
//oLocation = KLD_M.SaveCurrentLocation(KLD_M.oCurrentDoc); // Save cursor location/TR
  KLD_M.iNumberMarkers = KLD_M.CollectAllMarkers (KLD_M.aoMarkers); // fill the global array
  KLD_M.oCurrentMarker = KLD_M.aoMarkers[0];
  KLD_M.GetMarkerTypeListLocal (KLD_M.oCurrentDoc, KLD_M.asListMTypes);
  KLD_M.PaletteMarkers ();
  wPalM.g1.p1.g1.g2.et1.text = KLD_M.iNumberMarkers;
  KLD_M.ActivateButtonsM ("Nav");
  KLD_M.ButtonFirstM ();
//KLD_M.RestoreLocation (KLD_M.oCurrentDoc, oLocation); // Restore  cursor location/TR
} //--- end HandleAny -----------------------------------------------------------------------------

KLD_M.HandleIndex = function () { //===============================================================
/*            Handle index markers for documents
Called by     Command
Calling       xxx
History       2021-11-12
*/
//$.bp(true);
  KLD_M.PaletteIndex ();

} //--- end HandleIndex ---------------------------------------------------------------------------

KLD_M.DebugMenu = function () { //=================================================================
/*            Invoke individul menu items
Returns       Function value is xxx
Called by     Main
Calling       Command, FillItemList, FindReplInDoc, FindReplInBook
Comment       Used only during development - no more invoked in productive version
History       2020-12-29
*/
var aMenuItems, listIndex, txtIntro, wDlgDebug, sDebug_01, sDebug_02;

  aMenuItems = [
  "KLD_M.OpenHelpFile()",     // 0
  "KLD_M.HandleAny()",        // 1
  "KLD_M.HandleIndex()"];     // 2
  wDlgDebug = new Window("dialog",KLD_M.UItxt.DebugMenu.@msg_01.toString(),undefined);
  wDlgDebug.preferredSize.width = 450;
  wDlgDebug.location= [200,200];
  wDlgDebug.st1 = wDlgDebug.add ("statictext", undefined, KLD_M.UItxt.DebugMenu.@msg_02.toString(), {multiline:true});
  wDlgDebug.st1.preferredSize.width = 400;
	wDlgDebug.g1 = wDlgDebug.add ("group", undefined, undefined);
  	wDlgDebug.g1.l1 = wDlgDebug.g1.add ("listbox", undefined, undefined);
    wDlgDebug.g1.l1.preferredSize = [350,100];
  KLD_M.FillDlgList (wDlgDebug.g1.l1, aMenuItems);

  wDlgDebug.g1.l1.onDoubleClick = function () {
    listIndex = wDlgDebug.g1.l1.selection.valueOf();
    Command (listIndex + 1);
  };

  wDlgDebug.show();
} //--- end DebugMenu -----------------------------------------------------------------------------

KLD_M.OpenHelpFile = function () { // =============================================================
/*            Menu item "Documentation"
              Display the pdf with same name as this script ($.filename includes extension)
Called by     Command
Comment       No feedback in case the pdf does not exist => probably to be changed
*/
  if (KLD_M.oHelpFile.exists) {
    KLD_M.oHelpFile.execute();
  }
} //--- end OpenHelpfFile -------------------------------------------------------------------------

KLD_M.SetupFMmarkers = function () { //============================================================
/*            Initialise script
Returns       Function value is xxx
Called by     Main
Calling       GetScriptPath
History       2020-12-29
*/
var sHelpFile;
  KLD_M.sScriptDir  = KLD_M.GetScriptPath () + "\\" + KLD_M.APP_NAME;
  sHelpFile         = KLD_M.sScriptDir + "\\" + KLD_M.APP_NAME + ".pdf"
  KLD_M.oHelpFile   = File (sHelpFile);
//LogSettings ();
  return true;
} //--- end SetupFMmarkers -----------------------------------------------------------------------

KLD_M.GetScriptPath = function () { //=============================================================
/*            Get the script path
Returns       Function value is Path of the script
Called by     SetupFMmarkers, OpenTemplate, OpenPdfAtDestination
Calling       -
Attention     If sub-scripts are in a sub-directory then this function must be in the main module
History       2020-12-29
*/
var lastBSlash, sScriptPath, sWinPath;

  sScriptPath = $.fileName;                       // script path (including the file-name of the script)
                              // platform independent format e.g. /e/_DDDprojects/FM+EN-escript/escript
  sWinPath = new File (sScriptPath);
  sScriptPath = sWinPath.fsName;                  // get Windows format
  lastBSlash = sScriptPath.lastIndexOf("\\");     // \ needs escaping
  sScriptPath = sScriptPath.substring(0, lastBSlash);
  return sScriptPath;                             // no final \ !
} //--- end GetScriptPath -------------------------------------------------------------------------

/* <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  Script Registration and unregistration:                                                        <<
  The script is registered if it contains SetupNotifications and Notify functions                <<
  It stays registered until close of FM.                                                         <<
  Hence during development manual unregistration is necessary before each modification and       <<
  restart of the script.                                                                         <<
  This makes debugging cumbersome - hence I switch off this function during debugging            <<
*/ // <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

KLD_M.SetupNotifications = function () { //========================================================
/*            Set up the watch dogs
Called by     Main
*/
  Notification (Constants.FA_Note_PostActiveDocChange, true);
} //--- end SetupNotifications --------------------------------------------------------------------

KLD_M.RemoveNotifications = function () { //=====================================================
/*            Remove the watch dogs
Called by     Notify
Reference     Klaus Göbel in https://forums.adobe.com/thread/2174148
              Markus WiedeMaier in https://forums.adobe.com/thread/2276988
*/
  Notification (Constants.FA_Note_PostActiveDocChange, false);
} //--- end RemoveNotifications -----------------------------------------------------------------

function Notify (iNote, object, sParam, iParam) {  //==============================================
/*            Handle triggered events
Arguments     iNote           notification id to identify the event.
              object          current document or book object which generated the event.
              sParam          string, if any, associated with the notification.
              iParam          Integer, if any, associated with the notification. Probably the fcode or bit flags.
Called by     System
Calling       RemoveNotifications, UpdateDialogues, PaletteDocSettings
Attention     - This function must be named Notify and it can not set up as object
              - Writing to the console in the Notify function is pretty dangerous:
                Opening a panel in the course of Notify may also open the console window
                When then closing the console window (dismissing it) this will also close the panel
                and the panel ID is lost - subsequent reference to it is invalid!
Debugging     Not working in a Notify function: $.bp(…); $.writeln (…);
              These do work:                    Console (…); alert (…); Alert (…);
Comment       - FA_Note_PostActiveDocChange is triggered also by some Pods/dialgues, for example when inserting a table row
                It is also triggered by screen refresh (CTRL+L)
              - Revert to saved triggers 2, 105 and 29 in sequence. Hence 29 is superfluous here.
              - Open a document triggers 105 and 2 which will call UpdateDialogues twice => may create white screen
              - See discussion Trapping "white document window" at https://forums.adobe.com/thread/2276988
Reference     F_ApiNotification() in FDK Function Ref.
Used in       FMmarkers
History       2020-12-22
*/

  if (!object.ObjectValid()) {return;}            // when script is started from StartUp folder
  switch (iNote) {
// ---------------------------------------------- 105 active document has changed
    case Constants.FA_Note_PostActiveDocChange:
      KLD_M.oCurrentDoc = object;
      if (object.Name === "") {                   // new document
      }
      KLD_M.LastDocID = object.id                 // see FMcalc for appriate use of this <<<<<<<<<<
      break;
// ---------------------------------------------- unknown notification
    default:
      Console (KLD_M.Message ("e", KLD_M.UItxt.Notify.@msg_01, "Notify", [iNote]));
  }
} //--- end Notify --------------------------------------------------------------------------------

// <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
//$.bp(true);
KLD_M.Main ();                                    // follow the standard …
