﻿/* --- FMmarkers_Ancillary.jsx ------ UTF-8 -------------------------------------------------------
  to be included in FMmarkers.jsx
  This module should include only general purpose functions
  see E:\_DDDprojects\_JSX-library
*/

KLD_M.BackupSettings = function () { // ===========================================================
/*            Backup the settings file - it may be modified in this session
Returns       Function value is xxx
Called by     FMMarkers-PalM
Calling       Message
Comment       The settings file must be located in the same directory as the script
              The backup file will get the name xxx_backup.xml
History       2021-09-21
*/
var bOK, fBackup, fXmlFile, msg_01, sXmlFile, sBackup;

  sXmlFile = KLD_M.APP_NAME + ".xml";             // FMfindRepl.xml
  sBackup  = KLD_M.APP_NAME + "_backup.xml";
  fXmlFile = File($.fileName.replace (/[^\\\/]+$/i , sXmlFile));
  fBackup  = new File($.fileName.replace (/[^\\\/]+$/i , sBackup));

  try {
      fXmlFile.copy(fBackup);
      return  true;
  } catch (e) {                                   // what kind of error can this be ?
      msg_01 = "Copy error on XML file «" + sXmlFile + "». Error:\n "+ e;
      KLD_M.Message ("E", msg_01, "BackupSettings");
      fXmlFile.close();
      return false;
  }
} //--- end BackupSettings ------------------------------------------------------------------------

KLD_M.ComposeMsg = function (msgTxt, aValues) { //==================================================
/*            Compose message with values
Arguments     msgText         String may conatin \n (newline), symbolic parameters %01 ... %09
                              \r (same as \n) and \t (tab) not working with Default layout behaviour
              aValues         Array of values or strings replacing the symbolic parameters
                              %01 … %09 (unused are skipped)
Returns       Returns composed message
Attention     If msgTxt is read in from xml (incorrectly) this may be of type xml.
Used in       FMmarkers
History       2021-11-19
*/
var nj, Vals;

  if (typeof msgTxt == "xml" ) {msgTxt = msgTxt.toString();}
// Following two replacements only neccessary for read-in values
  msgTxt = msgTxt.replace (/\\n/g , "\n");
  msgTxt = msgTxt.replace (/\\t/g , "\t");        // not working with ScriptUI Default layout behaviour

  if (aValues != undefined) {
    nVals = aValues.length
    if (nVals > 9) {
      nVals = 9;                                  // %10 would be interpreted as %01
    }
    for (j = 1; j <= nVals; j++) {
      msgTxt = msgTxt.replace ("%0" + j, aValues[j-1]);
    }
  }
  return msgTxt;
} //--- end ComposeMsg  ---------------------------------------------------------------------------

KLD_M.GetFindParameters = function (iFS, iFV, sSearch, iMode, bWord, bCase, bBack) { //============
/*            Set up search parameters accordingly
Arguments     iFS             Constants.FS_xx defining what to search. The following are not valid:
                               1 FS_FindText
                               2 FS_FindElementTag
                               3 FS_FindCharFmt - to complicated
                               7 not defined
                               8 find object specified by iFV
                              14 FS_FindCondTextNotInCondTags
                              15 FS_FindCustomizationFlags
                              16 FS_FindWrap â†’ top of code
                              17 FS_RegexFlavour
                              19 FS_FindSelection - not elaborated
              iFV             Constants.FV_xx search option. Values >15 are not valid
              sSearch         what to search for - used only for certain types
              iMode           0 Simple search
                              1 Wildcard search; excludes RegEx and simple Search
                              2 Regex search; excludes bWord and bBack; RegexFlavour is set to Perl
              bWord           true to constrain search to whole word
              bCase           true to consider case in search
              bBack           true to search backwards
Returns       Parameters for the find method
Comment       FS_FindWrap: A flag that determines whether the find operation wraps when it
              reaches the location where the search began.
              Default is True; the find operations wraps.
              If False, after reaching the location where the search began, the find operation
              returns an empty TextRange and  FA_errno is set to FE_NotFound (-95).
              â†’ This flag can not be used to stop search at end of document ...
Transition    Old system                          New system
              1  Any Marker                       8, 0
              2  Any Variable                     8, 6
              3  Marker of type sSearch           9, 0
              4  Character format sSearch         5, 0
              5  Any Anchored Frame               8, 7
              6  Any Table                        8, 9
              7  Any Footnote                     8, 8

Called by     FindSomething
Reference     FDK Function Reference, F_ApiFind(), https://forums.adobe.com/thread/961616
Used in       FMmngFindRepl
History       2020-12-01
*/
var bWild, bRegEx, findParms, jj, qFlags, oPropVal1, oPropVal2, oPropVal3, oPropVal4;

  bWild  = iMode == 1;
  bRegEx = iMode == 2;

  findParms = new PropVals();

  oPropVal1 = new PropVal() ; // --- NoWrap stops at the iniital find start ----- Const.FS_FindWrap
  oPropVal1.propIdent.num = Constants.FS_FindWrap ;
  oPropVal1.propVal.valType = Constants.FT_Integer;
  oPropVal1.propVal.ival = 0 ;                    // no wrap
  findParms.push(oPropVal1);

  oPropVal2 = new PropVal() ;
  if (iFS == 1) { // -------------------------------------------------------- Constants.FS_FindText OK
    oPropVal2.propIdent.num = Constants.FS_FindText;
    oPropVal2.propVal.valType = Constants.FT_String;
    oPropVal2.propVal.sval = sSearch;
  } else if (iFS == 3) { // ---------------------------------------------- Constants.FS_FindCharFmt NOK

    KLD_M.Message ("E", KLD_M.UItxt.@msg_04, "GetFindParameters");
  } else if (iFS == 4) { // ----------------------------------------------- Constants.FS_FindPgfTag OK
    oPropVal2.propIdent.num = Constants.FS_FindPgfTag;
    oPropVal2.propVal.valType = Constants.FT_String;
    oPropVal2.propVal.sval = sSearch;
  } else if (iFS == 5) { // ---------------------------------------------- Constants.FS_FindCharTag OK
    oPropVal2.propIdent.num = Constants.FS_FindCharTag;
    oPropVal2.propVal.valType = Constants.FO_CharFmt;
    oPropVal2.propVal.sval = sSearch;
  } else if (iFS == 6) { // --------------------------------------------- Constants.FS_FindTableTag OK
    oPropVal2.propIdent.num = Constants.FS_FindTableTag;
    oPropVal2.propVal.valType = Constants.FO_TblFmt;
    oPropVal2.propVal.sval = sSearch;
  } else if (iFS == 8) { // ----------------------------------------------- Constants.FS_FindObject OK
    oPropVal2.propIdent.num   = Constants.FS_FindObject;
    oPropVal2.propVal.valType = Constants.FT_Integer;              // OK: used in existing projects
    if (iFV == 0)         { oPropVal2.propVal.ival = Constants.FV_FindAnyMarker ;                // OK
    } else if (iFV == 1)  { oPropVal2.propVal.ival = Constants.FV_FindAnyXRef;                   // OK
    } else if (iFV == 2)  { oPropVal2.propVal.ival = Constants.FV_FindUnresolvedXRef;            // OK
    } else if (iFV == 3)  { oPropVal2.propVal.ival = Constants.FV_FindAnyTextInset;              // OK
    } else if (iFV == 4)  { oPropVal2.propVal.ival = Constants.FV_FindUnresolvedTextInset;       // OK
    } else if (iFV == 6)  { oPropVal2.propVal.ival = Constants.FV_FindAnyVariable;               // OK
    } else if (iFV == 7)  { oPropVal2.propVal.ival = Constants.FV_FindAnchoredFrame;             // OK
    } else if (iFV == 8)  { oPropVal2.propVal.ival = Constants.FV_FindFootnote;                  // OK
    } else if (iFV == 9)  { oPropVal2.propVal.ival = Constants.FV_FindAnyTable;                  // OK
    } else if (iFV == 10) { oPropVal2.propVal.ival = Constants.FV_FindAutomaticHyphen;           // OK - loc is behind
    } else if (iFV == 11) { oPropVal2.propVal.ival = Constants.FV_FindAnyRubi;                   // OK
    } else if (iFV == 12) { oPropVal2.propVal.ival = Constants.FV_FindPgfFormatOverride;         // OK
    } else if (iFV == 13) { oPropVal2.propVal.ival = Constants.FV_FindCharacterFormatOverride;   // OK
    } else if (iFV == 14) { oPropVal2.propVal.ival = Constants.FV_FindTableFormatOverride;       // OK
    } else if (iFV == 15) { oPropVal2.propVal.ival = Constants.FV_FindStyleFormatOverride;       // OK - selection not visible
    } else {
      KLD_M.Message (KLD_M.UItxt.@msg_02, "GetFindParameters", [iFV]);
      return null;
    }
  } else if (iFS == 9) { // ------------------------------------_---- Constants.FS_FindMarkerOfType OK
    oPropVal2.propIdent.num = Constants.FS_FindMarkerOfType;
    oPropVal2.propVal.valType = Constants.FT_String;
    oPropVal2.propVal.sval = sSearch;
  } else if (iFS == 10) { // ------------------------------------------ Constants.FS_FindMarkerText OK
    oPropVal2.propIdent.num = Constants.FS_FindMarkerText;
    oPropVal2.propVal.valType = Constants.FT_String;
    oPropVal2.propVal.sval = sSearch;
  } else if (iFS == 11) { // -------------------------------------- Constants.FS_FindXRefWithFormat OK
    oPropVal2.propIdent.num = Constants.FS_FindXRefWithFormat;
    oPropVal2.propVal.valType = Constants.FT_String;
    oPropVal2.propVal.sval = sSearch;
  } else if (iFS == 12) { // --------------------------------------- Constants.FS_FindNamedVariable OK
    oPropVal2.propIdent.num = Constants.FS_FindNamedVariable;
    oPropVal2.propVal.valType = Constants.FT_String;
    oPropVal2.propVal.sval = sSearch;
  } else if (iFS == 13) { // ---------------------------------- Constants.15 OK
    oPropVal2.propIdent.num = Constants.FS_FindCondTextInCondTags;
    oPropVal2.propVal.valType = Constants.FT_Strings;
    oPropVal2.propVal.ssval[0] = sSearch;         // "cond1"
//  for (jj = 0; jj < sSearch.length; jj++) {     // ["cond1", "cond2", â€¦] does not work in ES
//    oPropVal2.propVal.ssval[jj] = sSearch[jj];
//  }
  } else if (iFS == 18) { // -------------------------------------------- Constants.FS_FindStyleTag OK - selection not visible
    oPropVal2.propIdent.num = Constants.FS_FindStyleTag;
    oPropVal2.propVal.valType = Constants.FT_String;
    oPropVal2.propVal.sval = sSearch;
  } else if (iFS == 19) { // ------------------------------------------- Constants.FS_FindSelection NOK
    KLD_Z.Message ("E", KLD_M.UItxt.@msg_03, "GetFindParameters");
  } else {
    KLD_Z.Message ("E", KLD_M.UItxt.@msg_01, "GetFindParameters", [iFS]);
    return null;
  }
  findParms.push(oPropVal2);

  if (bWord || bCase || bBack || bRegEx || bWild) { // customisation flags are ORED - OK 2020-12-09
    oPropVal3 = new PropVal() ;
    oPropVal3.propIdent.num = Constants.FS_FindCustomizationFlags ;
    oPropVal3.propVal.valType = Constants.FT_Integer;
    qFlags = 0;
    if (bCase)  { qFlags = qFlags   | Constants.FF_FIND_CONSIDER_CASE;}                     // 0x01
    if (bRegEx) { qFlags = qFlags   | Constants.FF_FIND_USE_REGEX;                          // 0x10
        oPropVal4 = new PropVal ();
        oPropVal4.propIdent.num = Constants.FS_RegexFlavour;                                // 17
        oPropVal4.propVal.valType = Constants.FT_Integer;
        oPropVal4.propVal.ival = Constants.FR_USE_PERL;                                     // 1
        findParms.push(oPropVal4);
    } else {
      if (bWild)  { qFlags = qFlags | Constants.FF_FIND_USE_WILDCARDS;}                     // 0x04
      if (bWord)  { qFlags = qFlags | Constants.FF_FIND_WHOLE_WORD;}                        // 0x02
      if (bBack)  { qFlags = qFlags | Constants.FF_FIND_BACKWARDS;}                         // 0x08
    }
    oPropVal3.propVal.ival = qFlags;              // qFlags OK; ...ival reports error
    findParms.push(oPropVal3);
  }
return findParms;
} //---  end GetFindParameters --------------------------------------------------------------------

KLD_M.GetPage = function (oDoc, obj) { // =========================================================
/*            Get the page with the object on it
Arguments     oDoc            Document to be handled, not necessarily the current document
              obj             Object for which to find the page
Returns       Function value =  object oPage
Called by     DisplayObject, etc.
Comment       The graphic object math must be written with lower case initital
              FDK lists FO_Math, ESTK lists Math, but an object inspection reveils it to be math
Reference     GetPage by Rick Quatro, https://forums.adobe.com/thread/2424187
Used in       FMvars, FMfindRepl
History       2021-03-26
*/
var oFrame = 0, cell = 0, objType = "", oProp = 0;

  while (obj) {
    oFrame = obj;
    objType = obj.constructor.name;
    switch (objType) {
      case "SubCol" :
        obj = obj.ParentTextFrame;
        break;
      case "Tbl" :
        obj = obj.FirstRowInTbl.FirstCellInRow;
        break;
      case "Row" :
        obj = obj.FirstCellInRow;
        break;
      case "Cell" :
      case "Pgf" :
      case "AFrame" :
        obj = obj.InTextFrame;
        break;
      case "TextLine" :
      case "TextFrame" :
      case "UnanchoredFrame" :
      case "Arc" :
      case "Ellipse" :
      case "Group" :
      case "Inset" :
      case "Line" :
      case "math" :
      case "Polygon" :
      case "Polyline" :
      case "Rectangle" :
      case "RoundRect" :
        if (obj.FrameParent.ObjectValid()) {
            obj = obj.FrameParent;
        } else {
            obj = 0;
        }
        break;
      case "XRef" :
      case "Var" :
        oProp = oDoc.GetTextPropVal (obj.TextRange.beg, Constants.FP_InTextObj);
        obj = oProp.propVal.obj;
        break;
      default:                                    // Prevent endless loop if unknown object type is found.
        obj = 0;
        break;
    }
  }
  if (oFrame) {
    return oFrame.PageFramePage;
  } else {
    return 0;
  }
} // --- end GetPage ------------------------------------------------------------------------------

KLD_M.GetXMLdata = function (sXmlFile) { // =======================================================
/*            Read localisation strings from xml file
Argements     sXmlFile        fully qualified path of xml file or just the file name.
              File must exist
              If no filepath is given (no slash or backslash present),
              then the file is assumed in the same directory as the script
Returns       xml sructure
Usage         xSettings = getXMLdata (sXmlFile);
Called by     SwitchUIlanguageX
Attention     this function is called by SwithUIlanguageX, hence localisation has not finished yet.
              Therefore: error messages need to be localised with the ExtendScript method
Reference     E:\FM-ExtendScript\SaveBookOldVersion\SaveBookAsOldVersion.jsx by Rick Quatro
History       2020-11-11
*/
var e, fXmlFile, xData, msg_01, msg_02;

  msg_01 = { en: "XML file «" + sXmlFile + "» not found.",
             de: "XML datei «" + sXmlFile + "» nicht gefunden.",
             fr: "Fichier XML «" + sXmlFile + "» non trouvé."};
  msg_02 = { en: "Read error on XML file «" + sXmlFile + "». Error:\n "+ e,
             de: "Lesefehler auf XML datei «" + sXmlFile + "» . Fehler:\n "+ e,
             fr: "Erreur de lecture sur le fichier XML «" + sXmlFile + "». Erreur:\n " + e};

  if ((sXmlFile == null) || (sXmlFile == undefined)) {
    return false;
  } else if (sXmlFile.indexOf("/") > 0 || sXmlFile.indexOf("\\") > 0) { // fully qualified path
    fXmlFile = File(sXmlFile);
  } else {
    fXmlFile = File($.fileName.replace (/[^\\\/]+$/i , sXmlFile));  // file in script folder
  }
  if (fXmlFile.exists === false) {
      KLD_G.IconMessage ("E", localize (msg_01), "getXMLdata");
      return false;
  }
  fXmlFile.open("r");
  fXmlFile.encoding = "UTF-8";
  try {
      xData = new XML(fXmlFile.read());
      fXmlFile.close();
      return  xData;
  } catch (e) {                                   // what kind of error can this be ?
      KLD_G.IconMessage ("E", localize (msg_02), "getXMLdata");
      fXmlFile.close();
      return false;
  }
} //--- end getXMLdata ----------------------------------------------------------------------------

KLD_M.IsInArray = function (array, item) { //======================================================
/*            Get index of item in array
Arguments     array           Array of items (strings, other)
              item            Item to be found in array
Returns       Function value is index or null
Called by     GetMarkerTypeListLocal
Attention     Match of item type and array item types not checked!
Used in       FMbibilo, FMcalc, FMvars, FMfindRepl
History       < 2021-02-22
*/
var j, jMax;
//$.bp(true);
  jMax = array.length;
  for (j = 0; j < jMax; j++) {
    if (array [j] == item ) { return j;}
  }
  return null;                                    // not found in array
} //--- end IsInArray -----------------------------------------------------------------------------

KLD_M.Message = function (msgType, msgTxt, msgTitle, aValues) { // ================================
/*            Compose message in alert box or return as function value
Arguments     msgType         (1 character): I (info), any other character is interpreted as E (Error)
                              for lower case characters the message is composed only and returned
              msgText         String may conatin \n (newline), symbolic parameters %01 ... %09
                              \r (same as \n) and \t (tab) not working with Default layout behaviour
              msgTitle        Title for the window
              aValues         Array of values or strings replacing the symbolic parameters %01 â€¦ %09 (unused are skipped)
Returns       msgType I, E    Alert message is issued with title
                      i, e    Composed message (no title available)
Attention     If msgTxt is read in from xml (incorrectly) this may be of type xml.
History       2020-11-18 test with VSCode and ESDebugger
              2021-08-11 msgTitle to be an own argument
*/
var bInline, j, nVals;

  if (typeof msgTxt == "xml" ) {msgTxt = msgTxt.toString();}
// Following two replacements only neccessary for read-in values, however they do not harm direct string parameters
  msgTxt = msgTxt.replace (/\\n/g , "\n");
  msgTxt = msgTxt.replace (/\\t/g , "\t");        // not working with ScriptUI Default layout behaviour
  if (msgType.match(/[a-z]/)) {                   // inline
    bInline = true;
  } else {
    bInline = false;
  }
  if (aValues != undefined) {
    nVals = aValues.length
    if (nVals > 9) {
      nVals = 9;                                  // %10 would be interpreted as %01
    }

    for (j = 1; j <= nVals; j++) {
      msgTxt = msgTxt.replace ("%0" + j, aValues[j-1]);
    }
  }
  if (bInline) {                                  // text for further use
    return msgTxt;
  }
  if (msgType.toUpperCase() == "I") {             // Alert message
    alert (msgTxt, msgTitle, false);
  } else {
    alert (msgTxt, msgTitle, true);
  }
} // --- end Message ------------------------------------------------------------------------------

KLD_M.ReplaceInString = function (string, sSearch, sReplace, bCase, bWord, bRegEx) { // ===========
/*            Find substring with options and replace it
Arguments     string          String in which to search
              sSearch         String to be searched
              sReplace        String replacing the search
              bCase           true for case sentitive search
              bWord           true so search whole words only
              bRegEx          true to interprete sSearch as RegEx
Returns       The string with the replacement
Called by     FMmarkers: ButtonReplace
History       2021-10-18
*/
var eReg0, eReg1, eReg2, eReg2, eReg3, eRegR, sResult, sFind;

  if (string === undefined) { return; }
  if (!bRegEx) {
    sFind = sSearch.replace (/([\\\^\$\.\|\?\*\+\(\)\[\{])/g,"\\$1"); // escape the 12 syntactical chars
  }

  eReg0 = new RegExp (sFind, "i");                // case insesitive, no words
  eReg1 = new RegExp (sFind);                     // case sensitive, no words
  eReg2 = new RegExp (sFind + "\\b", "i");        // case insesitive, word
  eReg3 = new RegExp (sFind + "\\b");             // case sensitive, word

  if (bRegEx) {
    eRegR = new RegExp (sSearch);                 // interprete as RegEx
    sResult = string.replace (eRegR, sReplace);
  } else {
    if (!bCase && !bWord) {
      sResult = string.replace (eReg0, sReplace);
    } else if (bCase && !bWord) {
      sResult = string.replace (eReg1, sReplace);
    } else if (!bCase && bWord) {
      sResult = string.replace (eReg2, sReplace);
    } else {
      sResult = string.replace (eReg3, sReplace);
    }
  }
  return sResult;
} //--- end ReplaceInString -----------------------------------------------------------------------

KLD_M.RestoreLocation = function (oDoc, oLocation) { // ===========================================
/*            Restore currunt location (current do) to previously saved one
Arguments     oDoc            Document to be handled, not necessarily the current document
              oLocation       object created by SaveCurrentLocation
Called by     xxx
Attention     oLocation must not be a global variable - or a stack must be established
              Not tested what happens if this document is no more open at restore
Reference     Rick Quatro at https://forums.adobe.com/thread/2243878
Used in       FMbiblio, FMcalc, FMvars
*/
var oPgf, tLoc, textRange;

  oDoc.CurrentPage = oLocation.page;
  oPgf = oLocation.pgf;
  if (oPgf.ObjectValid () === 1) {
    oDoc.TextSelection =   oLocation.txtRange;
  }
  oDoc.ScrollToText(textRange);
} //--- end RestoreLocation -----------------------------------------------------------------------

KLD_M.SaveCurrentLocation = function (oDoc) { // ==================================================
/*            Save the currunt location (current document) for later restore
Arguments     oDoc            Document to be handled, not necessarily the current document
Returns       Object to be used by RestoreLocation
Attention     Not tested what happens if this document is no more open at restore
Reference     Rick Quatro at https://forums.adobe.com/thread/2243878
Used in       FMbiblio, FMcalc, FMvars, FMgraph, FMnote
*/
var oLocation = {}; //, oTS;

//oTS = oDoc.TextSelection.beg.obj
  oLocation.pgf      = oDoc.TextSelection.beg.obj;
  oLocation.txtRange = oDoc.TextSelection;
  oLocation.page     = oDoc.CurrentPage;
  return oLocation;
} //--- end SaveCurrentLocation -------------------------------------------------------------------

KLD_M.SearchInString = function (string, iStart, sSearch, bCase, bWord, bRegEx) { // ===============
/*            Find substring with options
Arguments     string          String in which to search
              iStart          Index in string where to start the search
              sSearch         String to be searched
              bCase           true for case sentitive search
              bWord           true so search whole words only
              bRegEx          true to interprete sSearch as RegEx
Returns       The index of the found sSearch in string
Called by     FMmarkers: ButtonSearch, ButtonReplace
History       2021-10-18
*/
var eReg0, eReg1, eReg2, eReg2,  eReg3, eRegR, indx, sFind, sNext;

  if (string === undefined) { return; }
  sNext = string.substr (iStart);
  if (!bRegEx) {
    sFind = sSearch.replace (/([\\\^\$\.\|\?\*\+\(\)\[\{])/g,"\\$1"); // escape the 12 syntactical chars
  }

  eReg0 = new RegExp (sFind, "i");                // case insesitive, no words
  eReg1 = new RegExp (sFind);                     // case sensitive, no words
  eReg2 = new RegExp (sFind + "\\b", "i");        // case insesitive, word
  eReg3 = new RegExp (sFind + "\\b");             // case sensitive, word

  if (bRegEx) {
    eRegR = new RegExp (sSearch);                 // interprete as RegEx
    indx = sNext.search (eRegR);
  } else {
    if (!bCase && !bWord) {       indx = sNext.search (eReg0);
    } else if (bCase && !bWord) { indx = sNext.search (eReg1);
    } else if (!bCase && bWord) { indx = sNext.search (eReg2);
    } else {                      indx = sNext.search (eReg3);
    }
  }
  return indx; // + iStart;
} //--- end SearchInString ------------------------------------------------------------------------

KLD_M.SetDisplay  = function (bWanted) { // =======================================================
/*            Set display to on or off
Arguments     bWanted: true => set display ON; false => display OFF
Atention      Switching visibility to often lets this function fail.
              See https://forums.adobe.com/thread/2276988
Comment       An alternative to Redisplay() is the following, but is overkill
                oDoc = app.ActiveDoc
                oMasterPage = oDoc.FirstMasterPageInDoc;
                oTextFrame = oDoc.NewTextFrame (oMasterPage.PageFrame);
                oTextFrame.Delete ();
Reference     Rick Quattro
Used in       FMbiblio, FMcalc, FMvars, ...
*/
  if (bWanted == undefined) bWanted = true;
  if (app.Displaying != bWanted) {
    app.Displaying = bWanted;
  }
  if (app.Displaying) {                           // assure proper display
    app.ActiveDoc.Redisplay();                    // Redisplay, not ReDisplay as documented!
  }
} //--- end SetDisplay ----------------------------------------------------------------------------

KLD_M.SwitchUIlanguageX = function (sLangFile) { // ===============================================
/*            Read the language relevant XML file to fill the global array of texts
Arguments     sLangFile       Base of the languaage xml file. will be appended
                              _de.xml etc.
                              For test pupose this may be the beginning of a real path name
Returns       Values in global XML object KLD_M.UItxt.
Called by     Main
Calling       GetXMLdata
Comment       I do not convert everything to strings, but use toString() where required
Used in       FMfindRepl
History       2020-11-16 test with VSCode and ESDebugger
*/
var appLang, sLangCode;

  appLang = app.Language;
//appLang = Constants.FV_LANG_GERMAN;             // set explictly for UI test
	switch (appLang) {
		case Constants.FV_LANG_ENGLISH:               // 0x01
		case Constants.FV_LANG_BRITISH: {             // 0x02
			KLD_M.UItxt = KLD_M.GetXMLdata (sLangFile + "_en.xml");
			sLangCode = "en";
			break;
		}
		case Constants.FV_LANG_GERMAN: {              // 0x03
			KLD_M.UItxt = KLD_M.GetXMLdata (sLangFile + "_de.xml");
			sLangCode = "de";
			break;
		}
		case Constants.FV_LANG_FRENCH: {              // 0x05
			KLD_M.UItxt = KLD_M.GetXMLdata (sLangFile + "_fr.xml");
			sLangCode = "fr";
			break;
		}
		default: {
			KLD_M.UItxt = KLD_M.GetXMLdata (sLangFile + "_en.xml");
			sLangCode = "en";
		}
	}
	return sLangCode;
} //--- end SwitchUIlanguageX ---------------------------------------------------------------------

KLD_M.WriteXMLdata = function (xmlData, sXmlFile) { // ============================================
/*            Write XML data to file
Argements     xmlData         the full structure to be written.
              sXmlFile        fully qualified path of XML file or just the file name
                              If no filepath is given (no slash or backslash present),
                              then the file is assumed in the same directory as the script.
                              The file is created anew in UTF-8 with BOM
Reference     https://www.davidebarranca.com/2013/03/presets-management-with-dropdownlist-and-xml-in-extendscript/
              Klaus Göbel in https://community.adobe.com/t5/framemaker/xml-and-unicode-a-quarrelling-couple/td-p/11677704
History       2020-12-14
*/
var BOM = "\uFEFF", fXmlFile;

  if ((sXmlFile == null) || (sXmlFile == undefined)) {
    return false;
  } else if (sXmlFile.indexOf("/") > 0 || sXmlFile.indexOf("\\") > 0) { // fully qualified path
    fXmlFile = new File(sXmlFile);
  } else {
    fXmlFile = new File($.fileName.replace (/[^\\\/]+$/i , sXmlFile));  // file in script folder
  }
  try {
    fXmlFile.open('w');
    fXmlFile.encoding = "UTF-8";
    fXmlFile.write(BOM);
    fXmlFile.write(xmlData);
    fXmlFile.close();
  } catch (e) {
    KLD_M.Message ("E", KLD_M.UItxt.WriteXMLdata.@msg_01, "WriteXMLdata", [sXmlFile, e.message]);
    return false;
  }
  return true;
} // --- End WriteXMLdata -------------------------------------------------------------------------
