﻿/* --- FMmarkers-FunM.jsx ------ UTF-8 ------------------------------------------------------------
              Palette and Call-back functions, also ancillary function for the UI
Included by   FMgraph.jsx
Comment       Window ID must be set outside the dialogue function to be available for Notify functions
              It is set in modules FMgppp_Globals
History       2021-04-23
*/

KLD_M.DisplayInfoI = function (sText, sColour) { // ===============================================
/*            Display text in the information area
Arguments     sText           The text to be displayed. It may contain line breaks (CTRL+Enter)
              sColour         "black" or "red" - other colours are not defined.
Returns       Function value is xxx
Called by     PaletteMarkers
History       2021-09-21
*/
  wPalI.gGen.etInfo.text = sText;
  if (sColour == "red") {
    wPalI.gGen.etInfo.graphics.foregroundColor = wPalI.gGen.etInfo.graphics.newPen (wPalI.gGen.etInfo.graphics.PenType.SOLID_COLOR, [1,0,0], 1);
  } else {
    wPalI.gGen.etInfo.graphics.foregroundColor = wPalI.gGen.etInfo.graphics.newPen (wPalI.gGen.etInfo.graphics.PenType.SOLID_COLOR, [0,0,0], 1);
  }
} //--- end of DisplayInfoI -----------------------------------------------------------------------

KLD_M.DisplayInfo = function (sText, sColour) { // ===============================================
/*            Display text in the information area
Arguments     sText           The text to be displayed. It may contain line breaks (CTRL+Enter)
              sColour         "black" or "red" - other colours are not defined.
Returns       Function value is xxx
Called by     PaletteMarkers
History       2021-09-21
*/
  wPalM.gGen.etInfo.text = sText;
  if (sColour == "red") {
    wPalM.gGen.etInfo.graphics.foregroundColor = wPalM.gGen.etInfo.graphics.newPen (wPalM.gGen.etInfo.graphics.PenType.SOLID_COLOR, [1,0,0], 1);
  } else {
    wPalM.gGen.etInfo.graphics.foregroundColor = wPalM.gGen.etInfo.graphics.newPen (wPalM.gGen.etInfo.graphics.PenType.SOLID_COLOR, [0,0,0], 1);
  }
} //--- end of DisplayInfo -----------------------------------------------------------------------

KLD_M.CheckForMarker = function (oDoc, sMarkerName) { //===========================================
/*            Do such markers exist in document bodypages?
Arguments     oDoc            Document to be handled, not necessarily the current document
              sMarkerName     Type of marker to be found (not case sensitive) on body pages
Returns       The first marker of wanted type or undefined
Used in       FMcalc, FMmarkers
*/
var j, nTyped = 0, oMarker, oPage, sMarkerType;

  for (j = 0; j < KLD_M.iNumberMarkers; j++) {
    if (KLD_M.aoMarkers[j].MarkerTypeId.Name == sMarkerName) {
      nTyped++;
    }
  }
  return nTyped;
} //--- end CheckForMarker ------------------------------------------------------------------------

KLD_M.CollectAllMarkers = function (aoMarkers, sMarkerType) { // ==================================
/*            Gather all or particular types of marker-objects from the body pages, store in array
Arguments     aoMarkers       global array of markers to be filled
              sMarkerType     type of marker to be collected, "*" for all
Returns       Return number of markers collected
Called by     xxx
Calling       SaveCurrentLocation, GetFindParameters, RestoreLocation
Attention     To assure that markers are collected from all page types, the current page must be
              a body page. Othewise, when starting on a master page, only MP and XP are searched!
              This seems to be a quirk of the Find method.
Reference     KLaus Gobel, https://forums.adobe.com/message/8704549
*/
var aMarkerList = [], docStart, j, k, mListLen, nMarkers, oDoc, oLocation, oMarkerTI, oTextItem;

function FindTheStuff (docStart, aMarkerList) {   // --- Find markers on page type
var foundTextRange, tloc, locTextRange, oFindParams
  tloc = new TextLoc (docStart, 0);
  locTextRange = new TextRange (tloc, tloc);
  KLD_M.oCurrentDoc.TextSelection = locTextRange;
//                                      iFS, iFV, sSearch, iMode, bWord, bCase, bBack
  oFindParams= KLD_M.GetFindParameters (8,   0,   "",      0,     false, false, false); // finds only on body pages! <<<<<<<<<<<<<
  foundTextRange = KLD_M.oCurrentDoc.Find(tloc, oFindParams);
  while (foundTextRange.beg.obj.ObjectValid()) {
    aMarkerList.push(foundTextRange);
    tloc = foundTextRange.end;
    foundTextRange = KLD_M.oCurrentDoc.Find(tloc, oFindParams);
  }
} // --- end FindTheStuff ------------------------

  oDoc = KLD_M.oCurrentDoc;
  KLD_M.SetDisplay (false);                       // avoid flicker

  KLD_M.oCurrentDoc.CurrentPage = KLD_M.oCurrentDoc.FirstBodyPageInDoc;
  docStart = KLD_M.oCurrentDoc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf; // Body pages
  FindTheStuff (docStart, aMarkerList);

  KLD_M.oCurrentDoc.CurrentPage = KLD_M.oCurrentDoc.FirstMasterPageInDoc; // without this markers are found on body
  docStart = KLD_M.oCurrentDoc.FirstMasterPageInDoc.PageFrame.FirstGraphicInFrame.FirstPgf; // Master pages
  FindTheStuff (docStart, aMarkerList);

  KLD_M.oCurrentDoc.CurrentPage = KLD_M.oCurrentDoc.FirstRefPageInDoc; // without this markers are found on body
  docStart = KLD_M.oCurrentDoc.FirstRefPageInDoc.PageFrame.FirstGraphicInFrame.FirstPgf; // Reference pages
  FindTheStuff (docStart, aMarkerList);

  mListLen = aMarkerList.length;
  if (mListLen < 1) {
    KLD_M.RestoreLocation (KLD_M.oCurrentDoc, oLocation);// Restore  cursor location/TR
    KLD_M.SetDisplay (true);
    return null;                                  // common situation for new document
  }
  aoMarkers.length = 0;                           // clear before filling again.
  for (j = 0; j < mListLen; j++) {
    oMarkerTI = KLD_M.oCurrentDoc.GetTextForRange (aMarkerList[j], Constants.FTI_MarkerAnchor);
    for (k = 0; k < oMarkerTI.length; k++) {
      oTextItem = oMarkerTI[k];
      aoMarkers.push(oTextItem.obj);              // store the marker objects
    }
  }
  nMarkers = aoMarkers.length;
  KLD_M.SetDisplay (true);
  return nMarkers;                                // number of markers in array
} //--- end CollectAllMarkers ------------------------------------------------------------------------

KLD_M.DisplayMarker = function (oDoc, oCurrentMarker) { // ========================================
/*            Select marker to visualize the location of the current marker
              (Text Symbols must be on to see it)
Arguments     oDoc
              oCurrentMarker
Returns       -
Called by     ButtonInsertC, ButtonUpdateC, ButtonFirstC, ButtonPreviousC, ButtonNextC, ButtonLastC,
              ButtonRefreshC, ButtonInsertS, ButtonUpdateS, ButtonFirstS, ButtonPreviousS,
              ButtonNextS, ButtonLastS, ButtonRefreshS, EvaluateC, EvaluateS, CheckTermElementsC
*/
var myTextRange, oPage, oTextLoc1, oTextLoc2;

  if (oCurrentMarker.ObjectValid()) {
    oTextLoc1 = oCurrentMarker.TextLoc;
    oTextLoc2 = oCurrentMarker.TextLoc;
  } else {
    KLD_M.DisplayInfo (KLD_M.UItxt.DisplayMarker.@msg_01.toString(), "red");
    return false;
  }
  oTextLoc2.offset = oTextLoc2.offset+1;          // offset2 = offset1 is given by oTextLoc
  myTextRange  = new TextRange(oTextLoc1,oTextLoc2); // define a TextRange
  oDoc.TextSelection = myTextRange;               // if you want to select the marker
  oPage = KLD_M.GetPage (oDoc, oTextLoc1.obj);
  oDoc.CurrentPage = oPage;
  oDoc.ScrollToText(myTextRange);                 // requires a TextRange. NOT a location
} //--- end DisplayMarker -------------------------------------------------------------------------

KLD_M.FindMarkerByText = function (iMarker, search, sType) { // ====================================
/*            Find marker in global array by the text to be searched
Arguments     iMarker         Index of marker in array to start with
              search           Text to be found
              sType           Marker type, "*" for any marker
Returns       -
Called by     ButtonSearch
*/
var bCase, bWord, indx =  null, j, nMarkers, oMarker, sMarkerType, sMarkerText;

  bCase = wPalM.g1.p2.g5.cbCase.value;
  bWord = wPalM.g1.p2.g5.cbWord.value;
  bRegEx= wPalM.g1.p2.g5.cbRegEx.value;
  nMarkers = KLD_M.iNumberMarkers;
  for (j = iMarker; j < nMarkers; j++) {
    oMarker = KLD_M.aoMarkers[j];
    KLD_M.oCurrentMarker = oMarker;
    sMarkerType = oMarker.MarkerTypeId.Name;
    sMarkerText = oMarker.MarkerText;
    indx = KLD_M.SearchInString (sMarkerText, 0, search, bCase, bWord, bRegEx);
    if (sType != "*") {                           // search in particular marker
      if (sMarkerType != sType) {continue;}
    }
    if (indx >= 0) {                              // marker text found
      break;
    }
  }
  if (indx == -1) {
    KLD_M.DisplayInfo (KLD_M.ComposeMsg (KLD_M.UItxt.FindMarkerByText.@msg_01, [search]), "red");
    wPalM.g1.p2.g2.rbNavi.value = true;
    KLD_M.SearchMode (false);
    KLD_M.sMfilter = "*";
    wPalM.g1.p1.lbMtype.selection = null;
    return -1;
  }
  KLD_M.iFound = indx;
  wPalM.g1.p2.g3.et1.text = indx; //.toString(); // found in current marker on position
  KLD_M.DisplayMarker (KLD_M.oCurrentDoc, KLD_M.oCurrentMarker);
  wPalM.g1.p2.etMark.text = sMarkerText;
  KLD_M.sMarkerText = sMarkerText;
  KLD_M.sMarkerType = sMarkerType;
  KLD_M.iIndexOfMarker = j;                       // ???
  return j;                                       // index of marker
} //--- end FindMarkerByText -------------------------------------------------------------------------

KLD_M.GetIndexNearestMarker = function (aoMarkers, sMarkerType) { // ===============================
/*            Get nearest following marker
Arguments     aoMarkers       global array of markers
              sMarkerType     marker type to be found
Returns       Function value =  index in corresponding array
Called by     RefreshGlobals
Calling       GetFindParameters
Comment       On entry we are at oDoc.TextSelection and need to find the marker-index in the array.
              Have not found a method to stay at an alredy selected marker - switching between
              documents steps through the markers
Attention     This may require extension to honour sMarkerType <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
*/
var fRange, oFindParams, j, maLen, aMarkerRange = [], oTextItem, oTextLoc2, oThisMarker, nMarkers, thatId, thisId;

  oTextLoc2 = KLD_M.oCurrentDoc.TextSelection.beg; // at possible selection
  if (sMarkerType == "*") {
//                                        iFS, iFV, sSearch, iMode, bWord, bCase, bBack
    oFindParams= KLD_M.GetFindParameters (8,   0,   "",      0,     false, false, false); // any marker
  } else {
    oFindParams= KLD_M.GetFindParameters (9, 0, sMarkerType, 0,     false, false, false); // typed marker
  }
  fRange = KLD_M.oCurrentDoc.Find  (oTextLoc2, oFindParams);// finds text range (object pgf), lenght 1
  aMarkerRange = KLD_M.oCurrentDoc.GetTextForRange (fRange, Constants.FTI_MarkerAnchor);  // this is an array
  maLen = aMarkerRange.length;
  if (maLen === 0) {
    return null;                                  // no selection, assume 0 inc callin routine
  }
  for (j = 0; j < maLen; j++) {                   // loop should not be necessary due to findparms
    oTextItem = aMarkerRange[j];                  // superfluous ?
    oThisMarker = oTextItem.obj;                  // should contain only one marker
  }
  thisId = oThisMarker.Unique;
  nMarkers = aoMarkers.length;
  for (j = 0; j <  nMarkers; j++) {               // find the equivalent item in array
    thatId = aoMarkers[j].Unique;
    if (thisId == thatId) {break;}
  }
  return j;
} //--- end GetIndexNearestMarker -----------------------------------------------------------------

KLD_M.GetIndexOfMarker = function (aMarkers, oCurrentMarker) { // =================================
/*            Find index of current marker in glbl array
Arguments     aMarkers        global array of markers
              oCurrentMarker  marker to be found
Returns       Function value
Called by     RefreshGlobals
*/
var j, maLen, thatId, thisId;

  maLen = aMarkers.length;
  thisId = oCurrentMarker.Unique;
  for (j = 0; j <  maLen; j++) {                  // find the equivalent item in array
    thatId = aMarkers[j].Unique;
    if (thisId == thatId) {break;}
  }
  return j;
} //--- end GetIndexOfMarker ----------------------------------------------------------------------

KLD_M.GetMarkerType = function (oDoc, sMarkerName) { // ===========================================
/*            Get/define marker type
Arguments     oDoc            curent document
              sMarkerName     marker type to be handled
Called by     InsertMarker
Comment       If marker type not yet defined, define it
*/
var sMarkerType;

  sMarkerType = oDoc.GetNamedMarkerType (sMarkerName);
  if (sMarkerType.ObjectValid () === 0) {
    sMarkerType = oDoc.NewNamedMarkerType (sMarkerName);
  }
  return sMarkerType;
} //--- end GetMarkerType -------------------------------------------------------------------------

KLD_M.GetMarkerTypeListLocal = function (oDoc, asMarkerTypes) { // ==============================
/*            Fill array with list of Marker type names
Arguments     oDoc            current document
              asMarkerTypes   array to be filled with the Marker-types
Comment       This is an extended version of the JSXlib function for localised marker names:
              User defined markers are put at the beginning (with an introductory entry), then
              the system defined marker types (see list below) after an introductory line.
              Formats are found in the catalogue in an unpredictable order
History       2021-10-22
*/
var j, jLang, nSysTypes, nUsrTypes, sLangCodes = ["de", "en", "fr"], sUIlang, sMarkerType,
    sUserDef     = ["!!--- Benutzerdefiniert --!!", "!!----- User defined -----!!", "!!Défini par l'utilisateur!!"],
    sUserTypes   = [], sSysTypes = [];
    sSysTypes[0] = [                       // Source: fmres.dll - stringtable - 2189, 2190, 2191
      "!!----- FM-definiert -----!!", "Autor", "Bedingter Text", "Gleichung", "Glossar", "Glossareintrag",
      "HTML Macro", "Hypertext", "Index", "Kommentar", "Kopf-/Fußzeile $1", "Kopf-/Fußzeile $2",
      "Kopf-/Fußzeile $3", "Kopf-/Fußzeile $4", "Kopf-/Fußzeile $5", "Kopf-/Fußzeile $6",
      "Kopf-/Fußzeile $7", "Kopf-/Fußzeile $8", "Querverweis","Thema"]; // assumed: Thema = Terme du glossaire = GlossaryTerm
// FM-12 de: in place of Glossareintrag GlossaryTerm is defined - probably corrected later;
//           Thema / Subject / Sujet are the equivlents.
    sSysTypes[1] = [
      "!!------ FM defined -----!!", "Author", "Comment", "Conditional Text", "Cross-Ref",
      "Equation", "Glossary", "GlossaryTerm", "Header/Footer $1", "Header/Footer $2", "Header/Footer $3",
      "Header/Footer $4", "Header/Footer $5", "Header/Footer $6", "Header/Footer $7", "Header/Footer $8",
      "HTML Macro", "Hypertext", "Index", "Subject"];
    sSysTypes[2] = [
      "!!---- Défini par FM ----!!", "Auteur", "Commentaire", "En-tête/Pied page $1", "En-tête/Pied page $2",
      "En-tête/Pied page $3", "En-tête/Pied page $4", "En-tête/Pied page $5", "En-tête/Pied page $6",
      "En-tête/Pied page $7", "En-tête/Pied page $8", "Glossaire", "HTML_Macro", "Hypertexte", "Index",
      "RC", "Sujet", "Terme du glossaire", "Texte conditionnel"]; // RC → Références croisée
function fSort (a, b) {
  if      (a.toLowerCase() >  b.toLowerCase()) {return 1;}
  else if (a.toLowerCase() == b.toLowerCase()) {return 0;}
  else                       {return -1;}
} //--- end fSort --------------------------------

  sUIlang = KLD_M.APP_LANG;
  jLang = KLD_M.IsInArray (sLangCodes, sUIlang);
  nSysTypes = sSysTypes[0].length;
  asMarkerTypes.length = 0;                       // clear the array
  sUserTypes.push (sUserDef[jLang]);              // header line will sort on top

  sMarkerType = oDoc.FirstMarkerTypeInDoc;
  while(sMarkerType.ObjectValid() == true) {
    if (KLD_M.IsInArray (sSysTypes[jLang], sMarkerType.Name) != null) { // we know them already
      sMarkerType = sMarkerType.NextMarkerTypeInDoc;
      continue;
    }
    sUserTypes.push (sMarkerType.Name);           // add to user list
    sMarkerType = sMarkerType.NextMarkerTypeInDoc;
  }
  nUsrTypes = sUserTypes.length;
  sUserTypes.sort (fSort);                        // otherwise random order !
  for (j = 0; j < nUsrTypes; j++) {
    asMarkerTypes.push (sUserTypes[j]);
  }
  for (j = 0; j < nSysTypes; j++) {
    asMarkerTypes.push (sSysTypes[jLang][j]);
  }
} //--- end GetMarkerTypeListLocal ----------------------------------------------------------------

KLD_M.FillDlgList = function(oList, aListItems) { // ==============================================
/*            Fill dialogue list from global array
Arguments     oList           dialogue list to be filled from the array
              aListItems      array with the list items
Returns       -
Called by     various call-back functions
History       2021-09-21
*/
var j, nItems;

  nItems = aListItems.length;
  oList.removeAll ();                             // clear the list
  for (j = 0; j < nItems; j++) {
    oList.add ("item", aListItems[j]);
  }
} //--- end FillDlgList ---------------------------------------------------------------------------

KLD_M.InsertEditItem = function (oList, asItems, oEdit) { // ======================================
/*            Modify contents of edit field --- there are some of these...
Arguments     oList           List object providing the index (selected element)
              asItems         Array of items from which the text element is taken for insertion
              oEdit           EditText object (edit field in GUI).
                              The element .textselection may be a real selection or an empty string.
                              If the selection is empty, then the item from the list is inserted.
Returns       -
Called by     PaletteMarkers
History       2021-09-21
*/
var a, index, iTab, sItem, sSelection;
  wPalM.gGen.eInfotext = "";                       // clear info area
  if (oEdit != undefined || oEdit != null) {
    index = oList.selection.valueOf();
    sItem = asItems[index];
    sSelection = oEdit.textselection;
    if (oEdit.textselection == oEdit.text) {      // append at end of text
      oEdit.text = oEdit.text + sItem;
    } else {                                      // replace selection
      oEdit.textselection = sItem;
    }
    oList.selection.selected = false;
  } else {
    KLD_M.DisplayInfo (KLD_M.UItxt.InsertEditItem.@msg_01, "red");
  }
} // --- end InsertEditItem -----------------------------------------------------------------------

KLD_M.InsertMarker = function (oTextRange, sMarkerType, sMarkerText ) { // ========================
/*            Insert marker at location
Arguments     oTextRange      Where to insert the  marker. May be empty, defining only an Insertion Point
              sMarkerType     type of marker to be handled
              sMarkerText     contents of marker
Returns       The index of the inserted marker within the global marker array
Called by     ButtonInsertC, ButtonUpdateC
Calling       GetMarkerType, CollectAllMarkers
Comment       Nothing happens if sMarkerType not defined in oDoc
*/
var j, lenMarkerArray, oMarker, oTextLoc, oMarkerType, thatId, thisId;

  oTextLoc = oTextRange.end;                      // inserted at end of selection
  oMarkerType = KLD_M.GetMarkerType (KLD_M.oCurrentDoc, sMarkerType); // Get the specified oMarker type.
  if (oMarkerType.ObjectValid ()) {
      oMarker = KLD_M.oCurrentDoc.NewAnchoredMarker (oTextLoc);
      oMarker.MarkerTypeId = oMarkerType;
      oMarker.MarkerText   = sMarkerText;
      thisId = oMarker.Unique;                    // to find it in the updated array
  }
// --- update globals
  KLD_M.SetDisplay (false);                       // avoid flicker
  KLD_M.iNumberMarkers = KLD_M.CollectAllMarkers (KLD_M.aoMarkers); // preserves text location
  lenMarkerArray = KLD_M.aoMarkers.length;
  for (j = 0; j < lenMarkerArray; j++) {          // find the equivalent item in array
    thatId = KLD_M.aoMarkers[j].Unique;
    if (thisId == thatId) {break;}
  }
  KLD_M.SetDisplay (true);
  return j;                                       // index of oMarker in array
} //--- end InsertMarker --------------------------------------------------------------------------

KLD_M.ModifyMarker = function (oMarker, sNewMtype, sNewMtext ) { // ===============================
/*            Modify a marker in current document
Arguments     oMarker         Marker to be modified
              sNewMtype        Marker type the marker shall receive
              sNewMtext       Contents Marker the marker shall receive
Returns       Modified marker object or undefined if sNewMTye not defined in oDoc
Reference     Rick Quatro, https://community.adobe.com/t5/framemaker-discussions/
              way-to-quickly-replace-the-marker-text-in-all-same-custom-markers/m-p/12531991
*/
var oDoc, oNewMtype;

  oDoc     = app.ActiveDoc;
  if (sNewMtype != undefined) {
  	oNewMtype = oDoc.GetNamedMarkerType (sNewMtype);
    if (!oNewMtype.ObjectValid()) {return undefined; }
  	oMarker.MarkerTypeId = oNewMtype;
    return oMarker;
  }
  if (sNewMtext != undefined) {
    oMarker.MarkerText = sNewMtext;
    return oMarker;
  }
} //--- end ModifyMarker --------------------------------------------------------------------------
